"""Django Tilestache main settings"""
# coding: utf-8
import os
from django.conf import settings
from appconf import AppConf


class DjangoTilestacheSettings(AppConf):

    FILE_CONF = os.path.join(settings.BASE_DIR, 'tilestache.cfg')
    AUTOGENERATE_FILE = True
    CACHE = {'name': 'Test'}

    class Meta:
        # prefix for these variables
        # this will become TILESTACHE_FILE_CONF in your settings.py
        prefix = 'tilestache'
        proxy = True



# coding: utf-8
from TileStache import parseConfig
from .base import BaseConfiguration
from .utils import generate_coordinates
from .choices import DIRS_CHOICES


class BaseCacheConfig(BaseConfiguration):
    pass


class ExternalCacheConfig(BaseCacheConfig):

    """
    Configuration for external cache
    """

    klass = None
    arguments = None

    def is_valid(self):
        return self.klass is not None

    def _to_dict(self):
        dic = {'class': self.klass}
        if self.arguments:
            dic.update({'kwargs': self.arguments})
        return dic


class TestCacheConfig(BaseCacheConfig):

    """
    Configuration for test cache
    """

    verbose = True

    def _to_dict(self):
        return {
            'name': 'Test',
            'verbose': self.verbose
        }

    def is_valid(self):
        return isinstance(self.verbose, bool)


class DiskCacheConfig(BaseCacheConfig):

    """
    Configuration for disk cache
    """

    DIRS = [d[0] for d in DIRS_CHOICES]

    path = None
    umask = '0022'
    dirs = 'safe'
    gzip = ["txt", "text", "json", "xml"]

    def is_valid(self):
        return (self.path is not None and
                isinstance(self.path, basestring) and
                self.dirs in self.DIRS)

    def _to_dict(self):
        return {
            'name': 'Disk',
            'path': self.path,
            'umask': self.umask,
            'dirs': self.dirs,
            'gzip': self.gzip
        }


class MultiCacheConfig(BaseCacheConfig):

    """
    Configuration for multi cache
    """

    tiers = None

    def is_valid(self):
        if not self.tiers:
            return False
        return all([tier.is_valid() for tier in self.tiers])

    def _to_dict(self):
        tiers = [t.to_dict() for t in self.tiers]
        return {
            'name': 'Multi',
            'tiers': tiers
        }


class MemcacheCacheConfig(BaseCacheConfig):

    """
    Configuration for memcache cache
    """

    servers = None
    revision = 0
    key_prefix = ''

    def is_valid(self):
        # perhaps here we can do other validations
        return self.servers is not None

    def _to_dict(self):

        return {
            'name': 'Memcache',
            'servers': self.servers,
            'revision': self.revision,
            'key prefix': self.key_prefix
        }


class RedisCacheConfig(BaseCacheConfig):

    """
    Configuration for the redis cache
    """

    host = None
    port = 6379
    db = 0
    key_prefix = ''

    def is_valid(self):
        return self.host is not None and self.port is not None and self.db is not None

    def _to_dict(self):
        return {
            'name': 'Redis',
            'host': self.host,
            'port': self.port,
            'db': self.db,
            'key prefix': self.key_prefix
        }


class S3CacheConfig(BaseCacheConfig):

    """
    Configuration for S3 Cache
    """

    bucket = None
    access = None
    secret = None
    use_locks = True
    path = ''
    reduced_redundancy = False

    def is_valid(self):
        if self.access and not self.secret:
            return False

        if self.secret and not self.access:
            return False
        return self.bucket is not None

    def _to_dict(self):
        dic = {
            'name': 'S3',
            'bucket': self.bucket,
            'path': self.path,
            'use_locks': self.use_locks,
            'reduced_redundancy': self.reduced_redundancy
        }
        if self.access and self.secret:
            dic.update({
                'access': self.access,
                'secret': self.secret
            })
        return dic


class CacheManager(object):

    """
    Manages the Tilestache Cache.
    """

    ACTIONS = [
        '_seed_tile',
        '_purge_tile',
    ]

    def __init__(self, tilestache_cfg=None, **kwargs):
        if not tilestache_cfg:
            raise ValueError('tilestache_cfg is a required parameter')

        self.raw_config = tilestache_cfg
        # TODO: refatoar isto, pois pode ser um objeto também
        if isinstance(tilestache_cfg, basestring) or isinstance(tilestache_cfg, dict):
            self.config = parseConfig(tilestache_cfg)
        else:
            self.config = tilestache_cfg

    def _execute_action(self,
                        action,
                        layer,
                        geometry,
                        min_zoom=1,
                        max_zoom=21,
                        padding=0,
                        tile_format='PBF'):

        if not action:
            raise ValueError('action cannot be null')

        if action not in self.ACTIONS:
            raise ValueError('action must be one of %s' % ', '.join(self.ACTIONS.keys()))

        if not layer:
            raise ValueError('layer cannot be null')

        if isinstance(layer, basestring):
            tilestache_layer = self.config.layers.get(layer)
            if not layer:
                raise ValueError('layer %s is not available' % layer)
        else:
            tilestache_layer = layer

        if not geometry:
            # TODO: perhaps we can consider
            # that is the user does not provide an geometry
            # we will work on the entire cache
            raise ValueError('geometry cannot be null')

        action = getattr(self, action)
        tile_format = tile_format.upper()
        zooms = range(min_zoom, max_zoom + 1)
        coordinates = generate_coordinates(tilestache_layer,
                                           geometry,
                                           zooms,
                                           padding)

        for (offset, count, coord) in coordinates:
            action(tilestache_layer, coord, tile_format)

    def _seed_tile(self, layer, coord, tile_format):
        raise NotImplementedError('notimplemented')

    def _purge_tile(self, layer, coord, tile_format):
        '''removes a tile from the cache'''
        self.config.cache.remove(layer, coord, tile_format)

    def seed(self,
             layer,
             geometry,
             min_zoom=1,
             max_zoom=21,
             padding=0,
             tile_format='PBF'):
        self._execute_action(action='seed',
                             layer=layer,
                             geometry=geometry,
                             min_zoom=min_zoom,
                             max_zoom=max_zoom,
                             padding=0,
                             tile_format=tile_format)

    def purge(self,
              layer,
              geometry,
              min_zoom=1,
              max_zoom=21,
              padding=0,
              tile_format='PBF'):
        """
        purges the tilestache cache
        only inside the area of the
        geometry provided
        """
        self._execute_action(action='purge',
                             layer=layer,
                             geometry=geometry,
                             min_zoom=min_zoom,
                             max_zoom=max_zoom,
                             padding=0,
                             tile_format=tile_format)



# coding: utf-8
import requests
import logging
from TileStache import WSGITileServer
from apscheduler.schedulers.background import BackgroundScheduler
logger = logging.getLogger(__name__)


class RemoteConfigurationLoader(object):

    """
    This is a custom Tilestache config
    object.
    This will make requests to the appropriate
    endpoints
    and reconfigure the applicationt
    """

    cache = {}
    layers = {}
    dirpath = None

    def __init__(self, endpoint, auth=None, crash_tilestache=False):
        """
        based on a control
        """
        if not endpoint:
            raise ValueError('endpoint cannot be null')
        self.index = 'text/plain', 'Tilestache rules'
        self.endpoint = endpoint
        self.auth = auth
        self.crash_tilestache = crash_tilestache
        self.load_remote_config()

    def load_remote_config(self):
        try:
            logger.info('loading remote config at address %s', self.endpoint)
            response = self._fetch_remote_config(
                self.endpoint,
                self.auth
            )
            self._load_remote_config(response)
        except Exception as ex:
            logger.error('error while loading remote config %s', ex.message)
            if self.crash_tilestache:
                raise
            else:
                # if these are unset, they will be set
                # otherwise they'll keep the latest values
                if not hasattr(self, 'cache'):
                    self.cache = {}

                if not hasattr(self, 'layers'):
                    self.layers = {}

                if not hasattr(self, 'dirpath'):
                    self.dirpath = None

    def _setup_auth(self, auth):

        session = requests.Session()
        if auth and 'token' in auth:
            token = auth.get('token')
            session.headers.update({'Authorization': 'Token {0}'.format(token)})

        if auth and 'user' in auth and 'password' in auth:
            user = auth.get('user')
            password = auth.get('password')
            session.auth = (user, password)

        return session

    def _fetch_remote_config(self, endpoint, auth):
        if not endpoint:
            raise ValueError('you must provide the control endpoint')
        session = self._setup_auth(auth)

        return session.get(endpoint)

    def _load_remote_config(self, response):
        data = response.json()
        if 'cache' in data:
            self.cache = data['cache']

        if 'layers' in data:
            self.layers = data['layers']

        if 'dirpath' in data:
            self.dirpath = data['dirpath']


class RemoteTileStache(WSGITileServer, object):

    def __init__(self, endpoint, auth=None, config_timeout=120, crash_tilestache=False):

        """
        initializes a new tilestache server
        which reloads it's configuration every
        timeout
        """

        self.config = RemoteConfigurationLoader(
            endpoint=endpoint,
            auth=auth,
            crash_tilestache=crash_tilestache
        )
        self.config_timeout = config_timeout
        self.scheduler = BackgroundScheduler()
        self.job = self.scheduler.add_job(self.config.load_remote_config, 'interval', seconds=self.config_timeout)
        self.scheduler.start()
        super(RemoteTileStache, self).__init__(config=self.config, autoreload=False)



# coding: utf-8
from ModestMaps.Core import Coordinate
from ModestMaps.Geo import Location


def generate_coordinates(layer, geometry, zooms, padding=0):
    """ Generate a stream of (offset, count, coordinate) tuples for seeding.

        Flood-fill coordinates based on two corners, a list of zooms and padding.
    """
    # start with a simple total of all the coordinates we will need.
    if not layer:
        raise ValueError('layer argument is required')

    if not geometry:
        raise ValueError('geometry argument is required')

    if not zooms:
        zooms = range(1, 22)

    count = 0
    bounds = geometry_to_bounds(geometry)
    ul = layer.projection.locationCoordinate(bounds['nw'])
    lr = layer.projection.locationCoordinate(bounds['se'])
    for zoom in zooms:
        ul_ = ul.zoomTo(zoom).container().left(padding).up(padding)
        lr_ = lr.zoomTo(zoom).container().right(padding).down(padding)

        rows = lr_.row + 1 - ul_.row
        cols = lr_.column + 1 - ul_.column

        count += int(rows * cols)

    # now generate the actual coordinates.
    # offset starts at zero
    offset = 0

    for zoom in zooms:
        ul_ = ul.zoomTo(zoom).container().left(padding).up(padding)
        lr_ = lr.zoomTo(zoom).container().right(padding).down(padding)

        for row in range(int(ul_.row), int(lr_.row + 1)):
            for column in range(int(ul_.column), int(lr_.column + 1)):
                coord = Coordinate(row, column, zoom)

                yield (offset, count, coord)

                offset += 1


def geometry_to_bounds(geometry):

    """
    converts a django geometry to an envelope,
    and then converts this to TileStache's
    bounds
    """
    if not geometry:
        raise ValueError('geometry is required')

    if geometry.empty:
        raise ValueError('empty geometries cannot be converted')

    envelope = geometry.envelope
    if not envelope:
        raise ValueError('this geometry does not have an envelope and its invalid')

    # this is an envelope, so for sure we have 5 coordinates
    dic = {
        'south': envelope.coords[0][0][1],
        'west': envelope.coords[0][0][0],
        'north': envelope.coords[0][2][1],
        'east': envelope.coords[0][2][0]
    }
    northwest = Location(dic['north'], dic['west'])
    southeast = Location(dic['south'], dic['east'])

    return {
        'nw': northwest,
        'se': southeast
    }



# coding: utf-8
from rest_framework import serializers


class PolySerializer(object):

    """
    Simple instance that holds
    a few informations about
    who is the best serializer for this
    data
    """

    serializer_class = None
    can_serialize = None
    can_deserialize = None

    def __init__(self,
                 serializer_class=None,
                 can_serialize=None,
                 can_deserialize=None):

        if not serializer_class:
            raise ValueError('serializer_class cannot be null')
        if not can_serialize:
            raise ValueError('valid_obj cannot be null')
        if not can_deserialize:
            raise ValueError('valid_data cannot be null')

        self.serializer_class = serializer_class
        self.can_serialize = can_serialize
        self.can_deserialize = can_deserialize

    def can_serialize(self, data, context):
        """
        returns true if this object can be
        serialized with this serializer
        """
        if self.can_serialize:
            return self.can_serialize(data, context)

        return False

    def can_deserialize(self, data, context):
        """
        returns true if this object can be
        deserialized with this serializer
        """
        if self.can_deserialize:
            return self.can_deserialize(data, context)
        return False


class PolymorficSerializer(serializers.Serializer):

    """
    PolymorficSerializer can define
    a iterable of poly_serializers.
    They will be used to determine
    which serializer is suited to
    do the work on a particular object

    This itetable must hold instances
    of PolySerializer
    """

    poly_serializers = None

    def validate_poly_serializer(self):

        if not self.poly_serializers:
            raise ValueError('poly_serializers should not be null. it should be a iterable')

        return True

    def is_valid(self, raise_exception=False):
        self.validate_poly_serializer()

        suited = [p for p in self.poly_serializers if p.can_deserialize(data=self.initial_data, context=self.context)]
        if len(suited) <= 0:
            raise ValueError('no serializers are suitable')

        poly_serializer = suited.pop(0).serializer_class(data=self.initial_data, context=self.context)
        return poly_serializer.is_valid(raise_exception)

    def to_representation(self, obj):

        self.validate_poly_serializer()
        suited_serializers = [p for p in self.poly_serializers if p.can_serialize(obj, context=self.context)]
        if len(suited_serializers) <= 0:
            raise ValueError('no serializers are suitable for this object')
        poly_serializer = suited_serializers.pop(0).serializer_class(data=obj, context=self.context)
        return poly_serializer.to_representation(obj)

    def to_internal_value(self, data):

        self.validate_poly_serializer()

        suited = [p for p in self.poly_serializers if p.can_deserialize(data, context=self.context)]
        if len(suited) <= 0:
            raise ValueError('no serializers are suitable for this data')
        poly_serializer = suited.pop(0).serializer_class(data=data, context=self.context)
        return poly_serializer.to_internal_value(data)



# -*- coding: utf-8 -*-
from django.conf.urls import include, url
from rest_framework.routers import DefaultRouter
from django_tilestache.viewsets import LayerViewSet
from django_tilestache.views import (
    TileStacheConfiguration,
    TileStacheTile,
)

router = DefaultRouter(trailing_slash=False)
router.register(r'layers', LayerViewSet)

urlpatterns = [
    url(r'^api/', include(router.urls)),
    url(
        r'^api/tilestache/$',
        TileStacheConfiguration.as_view(),
        name='tilestache-configuration'
    ),
    url(
        r'^api/tilestache/(?P<layer_name>[-\w]+)/(?P<z>\d+)/(?P<x>\d+)/(?P<y>\d+).(?P<extension>\w+)',
        TileStacheTile.as_view(),
        name='tilestache-tile'
    ),
]



# coding: utf-8
from uuid import uuid4
from .exceptions import ConfigurationException


class BaseConfiguration(object):

    """
    Base configuration class.
    It supports creation using dictionaries
    that will auto assign values to instance
    members if the keys match.
    keys that do no exist on the instance
    will be discarded.
    Ideally all configurations will
    inherit this class.
    """

    optional_properties = tuple()

    id = None

    def __init__(self, **kwargs):
        if 'id' not in kwargs:
            self.id = uuid4()
        for k in kwargs.keys():
            if hasattr(self, k):
                setattr(self, k, kwargs[k])

    def is_valid(self):
        raise NotImplementedError('is_valid must be implemented.')

    def to_dict(self):
        if self.is_valid():
            return self._to_dict()

        raise ConfigurationException('The configuration for this object is not valid')

    def _to_dict(self):
        raise NotImplementedError('_serialize must be implemented.')

    def _merge_properties(self, output):
        for prop in self.optional_properties:
            if isinstance(prop, tuple):
                prop, key = prop[0], prop[1]
            else:
                prop, key = prop, prop
            if hasattr(self, prop):
                prop_value = getattr(self, prop)
                if prop_value:
                    output[key] = prop_value
        return output



# coding: utf-8
from django.utils.translation import ugettext_lazy as _


SAFE_DIRS_CHOICE = u'safe'
PORTABLE_DIRS_CHOICE = u'portable'

DIRS_CHOICES = (
    (SAFE_DIRS_CHOICE, _(u'Safe')),
    (PORTABLE_DIRS_CHOICE, _(u'Portable')),
)

PROVIDER_VECTOR = u'vector'
PROVIDER_MAPNIK = u'mapnik'
PROVIDER_PROXY = u'proxy'
PROVIDER_EXTERNAL = u'external'

PROVIDER_CHOICES = (
    (PROVIDER_VECTOR, u'Vector'),
    (PROVIDER_MAPNIK, u'Mapnik'),
    (PROVIDER_PROXY, u'Proxy'),
    (PROVIDER_EXTERNAL, u'External'),
)

VECTOR_DRIVER_SHP = u'shapefile'
VECTOR_DRIVER_POSTGRES = u'PostgreSQL'
VECTOR_DRIVER_ORACLE = u'Oracle'
VECTOR_DRIVER_MYSQL = u'MySQL'
VECTOR_DRIVER_SPATIALITE = u'Spatialite'
VECTOR_DRIVER_GEOJSON = u'GeoJSON'

VECTOR_DB_DRIVERS = (VECTOR_DRIVER_POSTGRES,
                     VECTOR_DRIVER_MYSQL,
                     VECTOR_DRIVER_ORACLE,)

VECTOR_FILE_DRIVERS = (VECTOR_DRIVER_SHP,
                       VECTOR_DRIVER_SPATIALITE,
                       VECTOR_DRIVER_GEOJSON,)

VECTOR_DRIVER_CHOICES = (
    (VECTOR_DRIVER_SHP, _(u'Shapefile')),
    (VECTOR_DRIVER_POSTGRES, _(u'PostgreSQL')),
    (VECTOR_DRIVER_ORACLE, _(u'Oracle')),
    (VECTOR_DRIVER_MYSQL, _(u'MySQL')),
    (VECTOR_DRIVER_SPATIALITE, _(u'SpatiaLite')),
    (VECTOR_DRIVER_GEOJSON, _(u'GeoJSON')),
)



# -*- coding: utf-8
from django.apps import AppConfig


class DjangoTilestacheConfig(AppConfig):
    name = 'django_tilestache'

    def ready(self):
        from .settings import DjangoTilestacheSettings  # noqa



# coding: utf-8
from rest_framework import viewsets
from django_tilestache.models import Layer
from django_tilestache.serializers import LayerSerializer


class LayerViewSet(viewsets.ModelViewSet):

    serializer_class = LayerSerializer
    queryset = Layer.objects.all()



# coding: utf-8
import copy
from rest_framework import serializers
from django_tilestache.polymorfic import (PolymorficSerializer,
                                          PolySerializer,)
from django_tilestache.models import Layer


class BoundsSerializer(serializers.Serializer):

    low = serializers.IntegerField(
        min_value=1,
        max_value=31
    )

    high = serializers.IntegerField(
        min_value=1,
        max_value=31
    )

    north = serializers.FloatField()

    west = serializers.FloatField()

    south = serializers.FloatField()

    east = serializers.FloatField()


class PreviewSerializer(serializers.Serializer):

    lat = serializers.FloatField(
        required=True
    )

    lon = serializers.FloatField(
        required=True
    )

    zoom = serializers.IntegerField(
        default=10
    )

    ext = serializers.CharField(
        max_length=16,
        default='png'
    )


class MetaTileSerializer(serializers.Serializer):

    rows = serializers.IntegerField(
        required=True,
        min_value=1
    )

    columns = serializers.IntegerField(
        required=True,
        min_value=1
    )

    buffer = serializers.IntegerField(
        required=True,
        min_value=0
    )


class BaseProviderSerializer(serializers.Serializer):

    name = serializers.CharField(max_length=64)

    def create(self, validated_data):
        self.data.update(validated_data)
        return self.data


class ProxyProviderSerializer(BaseProviderSerializer):

    url = serializers.CharField(
        max_length=512,
        required=True
    )

    timeout = serializers.IntegerField(
        required=False,
        min_value=1,
        max_value=60,
        allow_null=True
    )

    class Meta:
        fields = '__all__'


class ExternalProviderSerializer(BaseProviderSerializer):

    name = None  # removing the name fields, not needed

    klass = serializers.CharField(
        max_length=256,
        required=True,
        source='class'
    )

    kwargs = serializers.JSONField(
        required=False,
        allow_null=True
    )

    def _alias_field_name(self, data, original, changed):
        """
        this is a tremendous hack to maintain the
        attribute names on tilestache. external providers
        REQUIRE the name class, which is NOT a valid name
        for a DRF field
        """
        new_data = copy.deepcopy(data)
        if original in new_data:
            new_data[changed] = new_data[original]
            del new_data[original]
        return new_data

    def to_representation(self, obj):
        data = super(ExternalProviderSerializer, self).to_representation(obj)
        return self._alias_field_name(data, 'klass', 'class')

    def to_internal_value(self, data):
        data = self._alias_field_name(data, 'class', 'klass')
        return super(ExternalProviderSerializer, self).to_internal_value(data)

    class Meta:

        fields = ('klass', 'kwargs',)


class MapnikProviderSerializer(BaseProviderSerializer):

    mapfile = serializers.CharField(
        max_length=256,
        required=True
    )

    fonts = serializers.CharField(
        max_length=256,
        required=False,
        allow_null=True
    )

    scale_factor = serializers.FloatField(
        required=False,
        allow_null=True,
    )

    class Meta:

        fields = '__all__'


class VectorProviderSerializer(BaseProviderSerializer):

    driver = serializers.CharField(
        max_length=64,
        required=True
    )

    parameters = serializers.JSONField(
        required=True
    )

    properties = serializers.JSONField(
        required=False,
    )

    clipped = serializers.NullBooleanField(
        required=False,
    )

    projected = serializers.NullBooleanField(
        required=False,
    )

    precision = serializers.IntegerField(
        required=False,
        min_value=1,
        max_value=8
    )

    spacing = serializers.IntegerField(
        required=False
    )

    class Meta:
        fields = '__all__'


def can_serialize_vector(data, context):
    return 'name' in data and data['name'] == 'vector'


def can_serialize_proxy(data, context):
    return 'name' in data and data['name'] == 'proxy'


def can_serialize_mapnik(data, context):
    return 'name' in data and data['name'] == 'mapnik'


def can_serialize_external(data, context):
    return 'class' in data


class ProviderConfigSerializer(PolymorficSerializer):

    poly_serializers = (
        PolySerializer(ExternalProviderSerializer, can_serialize_external, can_serialize_external),
        PolySerializer(VectorProviderSerializer, can_serialize_vector, can_serialize_vector),
        PolySerializer(MapnikProviderSerializer, can_serialize_mapnik, can_serialize_mapnik),
        PolySerializer(ProxyProviderSerializer, can_serialize_proxy, can_serialize_proxy),
    )


class LayerSerializer(serializers.ModelSerializer):

    provider = ProviderConfigSerializer()

    metatile = MetaTileSerializer(
        required=False,
    )

    bounds = BoundsSerializer(
        required=False,
    )

    allowed_origin = serializers.CharField(
        max_length=256,
        required=False
    )

    preview = PreviewSerializer(
        required=False,
    )

    redirects = serializers.JSONField(
        required=False,
    )

    def create(self, validated_data):
        return Layer.objects.create(**validated_data)

    def update(self, instance, validated_data):

        nested_serializers = (
            ('provider', ProviderConfigSerializer),
            ('metatile', MetaTileSerializer),
            ('bounds', BoundsSerializer),
            ('preview', PreviewSerializer),
        )

        # this just iterates the nested serializers,
        # checks if there is data and it's valid.
        # if it's valid, set it back on the instance.
        # if it's not, it will raise an exception
        for key, serializer in nested_serializers:
            if key in validated_data:
                data = validated_data.pop(key)
                serializer = serializer(data=data)
                if serializer.is_valid(raise_exception=True):
                    setattr(instance, key, data)

        for k, v in validated_data.items():
            if hasattr(instance, k):
                setattr(instance, k, v)

        instance.save()
        return instance

    def _alias_field_name(self, data, original, changed):
        """
        this is a tremendous hack to maintain the
        attribute names on tilestache. external providers
        REQUIRE the name class, which is NOT a valid name
        for a DRF field
        """
        new_data = copy.deepcopy(data)
        if original in new_data:
            new_data[changed] = new_data[original]
            del new_data[original]
        return new_data

    def to_representation(self, obj):

        data = super(LayerSerializer, self).to_representation(obj)
        for k, v in data.items():
            if not v:
                del data[k]

        # this is here because we need allowed_origin to be allowed origin
        data = self._alias_field_name(data, 'cache_lifespan', 'cache lifespan')
        return self._alias_field_name(data, 'allowed_origin', 'allowed origin')

    class Meta:
        model = Layer
        fields = '__all__'



# -*- coding: utf-8 -*-
from django.utils.translation import ugettext_lazy as _
from django.contrib.gis.db import models
from jsonfield import JSONField


class Layer(models.Model):
    """
    Represents a tilestache layer
    """
    name = models.CharField(
        max_length=64,
        verbose_name=_(u'Name'),
        unique=True
    )

    provider = JSONField(
        verbose_name=_(u'Provider'),
        default=dict
    )

    projection = models.CharField(
        max_length=128,
        verbose_name=_(u'Projection'),
        null=True,
        blank=True
    )

    metatile = JSONField(
        verbose_name=_(u'Metatile'),
        null=True
    )

    state_lock_timeout = models.PositiveIntegerField(
        verbose_name=_(u'State Lock Timeout'),
        null=True
    )

    cache_lifespan = models.PositiveIntegerField(
        verbose_name=_(u'State Lock Timeout'),
        null=True
    )

    write_cache = models.NullBooleanField(
        verbose_name=_(u'Write Cache?'),
        null=True,
    )

    bounds = JSONField(
        verbose_name=_(u'Bounds'),
        null=True,
    )

    allowed_origin = models.CharField(
        max_length=256,
        verbose_name=_(u'Allowed Origin'),
        null=True
    )

    preview = JSONField(
        verbose_name=_(u'Preview'),
        null=True,
    )

    max_cache_age = models.PositiveIntegerField(
        verbose_name=_(u'Max Cache Age (seconds)'),
        null=True,
    )

    redirects = JSONField(
        verbose_name=_(u'Redirects'),
        null=True
    )

    tile_height = models.PositiveIntegerField(
        verbose_name=_(u'Tile Height'),
        null=True
    )

    def __unicode__(self):
        return self.name

    class Meta:

        verbose_name = _(u'Layer')
        verbose_name_plural = _('Layers')



# coding: utf-8
"""This module is responsible to hold out
query compilers. It's main goal is to hold
all the classes and objects responsible to
compile a specific query the dev chose
and write that as a TileStache query.

This uses the Django machinery to do
all that with some specifics
"""
from django.db import connections
from django.contrib.gis.db.models import (
    PointField,
    MultiPointField,
    LineStringField,
    MultiLineStringField,
    PolygonField,
    MultiPolygonField
)


class TileStacheQueryCompiler(object):

    GEOMETRY_FIELDS = (
        PointField,
        MultiPointField,
        LineStringField,
        MultiLineStringField,
        PolygonField,
        MultiPolygonField
    )

    BLACKLIST_FIELDS = (
        'id',
    )


class VecTilesQueryCompiler(TileStacheQueryCompiler):

    """given a certain model
    and a list of attributes,
    generates the query we need
    to feed tilestache with"""

    def select_attributes(self, queryset, attributes):
        model = queryset.model
        attrs = list()
        if isinstance(attributes, basestring):
            if attributes == '__all__':
                for field in model._meta.fields:
                    if field.name not in self.BLACKLIST_FIELDS and type(field) not in self.GEOMETRY_FIELDS:
                        attrs.append(field)
                return attrs
        else:
            for attr in attributes:
                field = model._meta.get_field(attr)
                if field.name not in self.BLACKLIST_FIELDS and type(field) not in self.GEOMETRY_FIELDS:
                    attrs.append(model._meta.get_field(attr))

            return attrs

        raise ValueError('attributes must be a string with __all__ or a iterable')

    def compile_geometry(self, queryset, geometry_field):
        model = queryset.model
        geometry_field = model._meta.get_field(geometry_field)
        name = geometry_field.name
        if geometry_field.srid != 900913:
            fmt = "ST_Transform({0}, 900913) as __geometry__"
        else:
            fmt = "{0} as __geometry__"
        return fmt.format(name)

    def compile_id(self, id_field=None):
        id_field = id_field or 'id'
        return '{0} as __id__'.format(id_field)

    def compile(self, queryset, attributes, id_field='id', geometry_field='geom', database='default'):
        id_query = self.compile_id(id_field)
        geom_query = self.compile_geometry(queryset, geometry_field)
        attrs = [attr.name for attr in self.select_attributes(queryset, attributes)]
        queryset = queryset.values(*attrs)
        query, params = queryset.query.sql_with_params()
        with connections[database].cursor() as cursor:
            if hasattr(cursor, 'mogrify'):
                query = cursor.mogrify(query, params)
            else:
                query = queryset.query.__str__()

        return query.replace(
            'SELECT ',
            "SELECT {0}, {1}, ".format(id_query, geom_query)    
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-05-08 18:37
from __future__ import unicode_literals

from django.db import migrations, models
import jsonfield.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Layer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64, unique=True, verbose_name='Name')),
                ('provider', jsonfield.fields.JSONField(default=dict, verbose_name='Provider')),
                ('projection', models.CharField(blank=True, max_length=128, null=True, verbose_name='Projection')),
                ('metatile', jsonfield.fields.JSONField(null=True, verbose_name='Metatile')),
                ('state_lock_timeout', models.PositiveIntegerField(null=True, verbose_name='State Lock Timeout')),
                ('cache_lifespan', models.PositiveIntegerField(null=True, verbose_name='State Lock Timeout')),
                ('write_cache', models.NullBooleanField(verbose_name='Write Cache?')),
                ('bounds', jsonfield.fields.JSONField(null=True, verbose_name='Bounds')),
                ('allowed_origin', models.CharField(max_length=256, null=True, verbose_name='Allowed Origin')),
                ('preview', jsonfield.fields.JSONField(null=True, verbose_name='Preview')),
                ('max_cache_age', models.PositiveIntegerField(null=True, verbose_name='Max Cache Age (seconds)')),
                ('redirects', jsonfield.fields.JSONField(null=True, verbose_name='Redirects')),
                ('tile_height', models.PositiveIntegerField(null=True, verbose_name='Tile Height')),
            ],
            options={
                'verbose_name': 'Layer',
                'verbose_name_plural': 'Layers',
            },
        ),
    ]



# coding: utf-8
import TileStache
from rest_framework.views import APIView
from rest_framework import status
from rest_framework.response import Response
from django.http import HttpResponse
from django_tilestache.configuration import (
    get_config,
    get_tilestache_config,
)


class TileStacheConfiguration(APIView):

    """
    View that outputs the current tilestache
    configuration
    """

    def get(self, request, format='json'):

        """Returns the configuration"""
        return Response(get_config())


class TileStacheTile(APIView):
    """
    renders tilestache tiles
    """
    def get(self, request, layer_name, z, x, y, extension):
        """
        Fetch tiles with tilestache.
        """
        try:
            extension = extension.upper()
            config = get_tilestache_config()
            path_info = "%s/%s/%s/%s.%s" % (layer_name, z, x, y, extension)
            coord, extension = TileStache.splitPathInfo(path_info)[1:]
            try:
                tilestache_layer = config.layers[layer_name]
            except:
                return Response({'status': 'layer not found'}, status=status.HTTP_404_NOT_FOUND)

            status_code, headers, content = tilestache_layer.getTileResponse(coord, extension)
            mimetype = headers.get('Content-Type')
            if len(content) == 0:
                status_code = 404

            response = HttpResponse(
                content,
                **{
                    'content_type': mimetype,
                    'status': status_code
                }
            )
            if hasattr(tilestache_layer, 'allowed origin'):
                response['Access-Control-Allow-Origin'] = tilestache_layer.get('allowed origin')
            return response
        except Exception as ex:
            return Response(
                {
                    'status': 'error',
                    'message': ex.message
                },
                status=status.HTTP_404_NOT_FOUND
            )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


def get_version(*file_paths):
    """Retrieves the version from django_tilestache/__init__.py"""
    filename = os.path.join(os.path.dirname(__file__), *file_paths)
    version_file = open(filename).read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')

version = '0.6.4'


if sys.argv[-1] == 'publish':
    try:
        import wheel
        print("Wheel version: ", wheel.__version__)
    except ImportError:
        print('Wheel library missing. Please run "pip install wheel"')
        sys.exit()
    os.system('python setup.py sdist upload')
    os.system('python setup.py bdist_wheel upload')
    sys.exit()

if sys.argv[-1] == 'tag':
    print("Tagging the version on git:")
    os.system("git tag -a %s -m 'version %s'" % (version, version))
    os.system("git push --tags")
    sys.exit()

readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

setup(
    name='django-tilestache',
    version=version,
    description="""Command and Control Center for Tilestache, inside a django app""",
    long_description=readme + '\n\n' + history,
    author='George Silva',
    author_email='george@sigmageosistemas.com.br',
    url='https://gitlab.sigmageosistemas.com.br/dev/django-tilestache',
    packages=[
        'django_tilestache',
    ],
    include_package_data=True,
    install_requires=[
        'django',
        'django-appconf',
        'tilestache',
        'jsonfield',
        'apscheduler',
        'djangorestframework',
        'djangorestframework-gis',
        'djangorestframework-filters',
        'requests',
        'pylru',
    ],
    license="MIT",
    zip_safe=False,
    keywords='django-tilestache',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)



# coding: utf-8
import os
import codecs
import json
import logging
from TileStache import parseConfig
from django_tilestache.caches import BaseCacheConfig
from django_tilestache.models import Layer
from django_tilestache.serializers import LayerSerializer
from django_tilestache.settings import settings
logger = logging.getLogger(__name__)


def get_config():

    """
    Gets a configuration
    """

    conf_man = ConfigurationManager()
    layers = Layer.objects.all()
    response = conf_man.build_config(cache=settings.TILESTACHE_CACHE,
                                     layers=layers)
    return response


def get_tilestache_config():

    """
    Gets the config for tilestache.
    this returns an object that can be used by
    tilestache and serve tiles.

    There is two methods, and while the method get_Config
    only gets a JSON representation, this one uses the former
    to construct a full TileStache object
    """
    return parseConfig(get_config())


class ConfigurationManager(object):

    LEFTOVER_FIELDS = ('id', )

    def read(self, path):
        pass

    def prepare_for_write(self, layer):
        """
        Prepare this layer for write.
        This means trimming up some extra
        fields that we have in Django, but
        we don't need them in tilestache
        """
        serializer = LayerSerializer(instance=layer)
        data = serializer.data
        for field in self.LEFTOVER_FIELDS:
            data.pop(field)
        name = data.pop('name')
        return {name: data}

    def build_config(self,
                     cache,
                     layers,
                     dirpath=None,
                     index=None):
        if not cache:
            cache_data = {
                'name': 'Test'
            }

        if isinstance(cache, dict):
            cache_data = cache

        if isinstance(cache, BaseCacheConfig):
            cache_data = cache.to_dict()

        layers_data = {}
        if layers:
            for layer in layers:
                layer_data = self.prepare_for_write(layer)
                layers_data.update(layer_data)

        return {
            'dirpath': dirpath,
            'cache': cache_data,
            'layers': layers_data
        }

    def serialize(self,
                  cache,
                  layers,
                  dirpath=None,
                  index=None,
                  indent=0):

        """
        Serializes a cache and layers config
        to json string
        """

        result = self.build_config(cache,
                                   layers,
                                   dirpath,
                                   index)
        return json.dumps(result, ensure_ascii=False, indent=indent)

    def write(self, path, cache, layers, dirpath=None, index=None, indent=0):
        """
        Writes a configuration to a location in disk
        indent must be an integer. it will be used
        as the number of spaces to pretty print the JSON
        """
        if not path:
            raise ValueError('path cannot be null')

        if not os.path.exists(os.path.dirname(path)):
            raise ValueError('directory %s does not exist.' % os.path.dirname(path))

        if index and not isinstance(index, tuple):
            raise ValueError('index must be two tuple')

        if not dirpath:
            dirpath = os.path.dirname(path)

        with codecs.open(path, 'w', 'utf-8') as cfg_file:
            cfg_file.write(self.serialize(
                cache,
                layers,
                dirpath=dirpath,
                index=index,
                indent=indent)
            )
